@@ -6,10 +6,22 @@ try:
     import json
 except ImportError:
     import simplejson as json
+try:
+    import configparser
+except ImportError:
+    import ConfigParser as configparser
+
+from functools import wraps
 from mock import Mock, patch
+from collections import namedtuple
+from os.path import dirname
+from sys import modules
+
+from txclib.exceptions import AuthenticationError, TXConnectionError
+from txclib.project import (Project, DEFAULT_PULL_URL)
 
-from txclib.project import Project, ProjectNotInit
 from txclib.config import Flipdict
+from txclib import utils
 
 
 class TestProject(unittest.TestCase):
@@ -19,32 +31,29 @@ class TestProject(unittest.TestCase):
         """Test _get_tx_dir_path function"""
         expected_path = '/tmp/'
         m_find_dot_tx.return_value = expected_path
-        p = Project(init=False)
-        path = p._get_tx_dir_path(path_to_tx=None)
+        path = utils.get_tx_dir_path(path_to_tx=None)
         self.assertEqual(path, expected_path)
         m_find_dot_tx.assert_called_once_with()
 
         expected_path = '/opt/'
-        path = p._get_tx_dir_path(path_to_tx=expected_path)
+        path = utils.get_tx_dir_path(path_to_tx=expected_path)
         self.assertEqual(path, expected_path)
         # make sure it has not been called twice
         m_find_dot_tx.assert_called_once_with()
 
     @patch('os.path.exists')
     def test_get_config_file_path(self, m_exists):
-        """Test _get_config_file_path function"""
-        p = Project(init=False)
+        """Test get_config_file_path function"""
         m_exists.return_value = True
-        p._get_config_file_path('/tmp/')
+        utils.get_config_file_path('/tmp/')
         m_exists.assert_called_once_with('/tmp/.tx/config')
 
         m_exists.return_value = False
-        with self.assertRaises(ProjectNotInit):
-            p._get_config_file_path('/tmp/')
+        with self.assertRaises(utils.ProjectNotInit):
+            utils.get_config_file_path('/tmp/')
 
-    @patch('txclib.utils.confirm')
     @patch('txclib.config.configparser')
-    def test_getset_host_credentials(self, m_parser, m_confirm):
+    def test_getset_host_credentials(self, m_parser):
         p = Project(init=False)
         # let suppose a token has been set at the config
         dummy_token = 'salala'
@@ -68,6 +77,76 @@ class TestProject(unittest.TestCase):
         self.assertEqual(username, 'username')
         self.assertEqual(password, 'passw0rdz')
 
+    @patch('txclib.project.input')
+    @patch('txclib.config.configparser')
+    def test_getset_host_credentials_no_transifexrc(
+            self, m_parser, m_input):
+        p = Project(init=False)
+        # let suppose a token has been set at the config
+        dummy_token = 'salala'
+        p.txrc = m_parser
+        p.save = Mock()
+        p.validate_credentials = Mock(return_value=True)
+        p.txrc_file = '/tmp'
+        p.txrc.get.side_effect = configparser.NoSectionError('test')
+        m_input.return_value = dummy_token
+        username, password = p.getset_host_credentials('test')
+        self.assertEqual(username, 'api')
+        self.assertEqual(password, dummy_token)
+        self.assertEqual(p.txrc.set.call_count, 4)
+        self.assertEqual(m_input.call_count, 1)
+        p.save.assert_called()
+
+    @patch('txclib.project.utils.confirm')
+    @patch('txclib.config.configparser')
+    def test_getset_host_credentials_update_transifexrc(
+            self, m_parser, m_input):
+        p = Project(init=False)
+        dummy_token = 'salala'
+        p.txrc = m_parser
+        p.save = Mock()
+        p.txrc_file = '/tmp'
+        p.validate_credentials = Mock(return_value=True)
+        p.txrc.get.side_effect = [
+            'foo', 'bar'
+        ]
+        # transifexrc does not get updated if credentials are the same
+        username, password = p.getset_host_credentials(
+            'test', username='foo', password='bar'
+        )
+        self.assertEqual(username, 'foo')
+        self.assertEqual(password, 'bar')
+        self.assertEqual(p.txrc.set.call_count, 0)
+        self.assertEqual(m_input.call_count, 0)
+        self.assertEqual(p.save.call_count, 0)
+
+        # transifexrc is not updated if confirm is no
+        p.txrc.get.side_effect = [
+            'foo', 'bar'
+        ]
+        m_input.return_value = False
+        username, password = p.getset_host_credentials('test',
+                                                       token=dummy_token)
+        self.assertEqual(username, 'api')
+        self.assertEqual(password, dummy_token)
+        self.assertEqual(p.txrc.set.call_count, 0)
+        self.assertEqual(m_input.call_count, 1)
+        self.assertEqual(p.save.call_count, 0)
+
+        # transifexrc is not updated if confirm is yes
+        p.txrc.get.side_effect = [
+            'foo', 'bar'
+        ]
+        m_input.return_value = True
+        m_input.reset_mock()
+        username, password = p.getset_host_credentials('test',
+                                                       token=dummy_token)
+        self.assertEqual(username, 'api')
+        self.assertEqual(password, dummy_token)
+        self.assertEqual(p.txrc.set.call_count, 4)
+        self.assertEqual(m_input.call_count, 1)
+        p.save.assert_called()
+
     def test_extract_fields(self):
         """Test the functions that extract a field from a stats object."""
         stats = {
@@ -569,6 +648,150 @@ class TestProjectPull(unittest.TestCase):
                 res = self.p._should_download('pt', self.stats, None, True)
                 self.assertEqual(res, True)
 
+    def test_get_url_by_pull_mode(self):
+        self.assertEqual(
+            'pull_sourceastranslation_file',
+            self.p._get_url_by_pull_mode(mode='sourceastranslation')
+        )
+        self.assertEqual(
+            DEFAULT_PULL_URL,
+            self.p._get_url_by_pull_mode(mode='invalid mode')
+        )
+        self.assertEqual(
+            DEFAULT_PULL_URL,
+            self.p._get_url_by_pull_mode(mode=None)
+        )
+
+    def fixture_mocked_project(func):
+        """A mock object with main os and http operations mocked"""
+        @wraps(func)
+        def wrapper(*args, **kwargs):
+            app_dir = dirname(modules['txclib'].__file__)
+            config_file = app_dir + "/../tests/templates/config"
+            transifex_file = app_dir + "/../tests/templates/transifexrc"
+            with patch("txclib.utils.encode_args") as mock_encode_args, \
+                patch("txclib.utils.determine_charset")\
+                    as mock_determine_charset, \
+                    patch("txclib.utils.get_transifex_file",
+                          return_value=transifex_file) \
+                    as mock_get_transifex_file, \
+                    patch("txclib.utils.get_config_file_path",
+                          return_value=config_file) \
+                    as mock_get_config_file_path, \
+                    patch("txclib.utils.save_txrc_file") \
+                    as mock_save_txrc_file, \
+                    patch("txclib.project.Project._get_stats_for_resource") \
+                    as mock_get_stats_for_resource:
+
+                # Create fake https response
+                def encode_args(*args, **kwargs):
+                    struct = namedtuple("response", "data status close")
+                    return struct(status=401, data="mock_response", close=Mock())
+                mock_determine_charset.return_value = "utf-8"
+                mock_encode_args.return_value = encode_args
+
+                # Mock configuration files
+                p = Project(init=False)
+                p._init(path_to_tx=app_dir + "/../templates")
+
+                kwargs['mock_project'] = p
+                kwargs['mocks'] = {
+                    'mock_determine_charset': mock_determine_charset,
+                    "mock_encode_args": mock_encode_args,
+                    "mock_get_config_file_path": mock_get_config_file_path,
+                    "mock_get_stats_for_resource": mock_get_stats_for_resource,
+                    "mock_get_transifex_file": mock_get_transifex_file,
+                    "mock_save_txrc_file": mock_save_txrc_file
+                }
+                return func(*args, **kwargs)
+        return wrapper
+
+    @fixture_mocked_project
+    @patch("txclib.project.logger.error")
+    def test_pull_raises_authentication_exception(self, mock_logger, **kwargs):
+        project = kwargs['mock_project']
+        with self.assertRaises(AuthenticationError):
+            project.pull()
+            mock_logger.assert_called_once_with(
+                Project.AUTHENTICATION_FAILED_MESSAGE)
+
+    @fixture_mocked_project
+    @patch('txclib.project.Project.do_url_request')
+    def test_pull_with_branch_pulls_from_right_resource(self, m, **kwargs):
+        m.return_value = ('{"i18n_type": "PO"}', '')
+        project = kwargs['mock_project']
+        project.pull(branch='somebranch')
+        self.assertDictEqual(project.url_info, {
+            'host': 'https://fake.com',
+            'project': 'example',
+            'resource': 'somebranch--enpo'
+        })
+
+    @fixture_mocked_project
+    @patch('txclib.project.Project.do_url_request')
+    def test_push_with_branch_pushes_to_right_resource(self, m, **kwargs):
+        m.return_value = ('{"i18n_type": "PO"}', '')
+        project = kwargs['mock_project']
+        project.push(source=True, branch='somebranch')
+        self.assertDictEqual(project.url_info, {
+            'host': 'https://fake.com',
+            'project': 'example',
+            'resource': 'somebranch--enpo'
+        })
+
+    @fixture_mocked_project
+    @patch('txclib.project.Project.do_url_request')
+    def test_push_with_branch_weird_characters_are_handled(self, m, **kwargs):
+        m.return_value = ('{"i18n_type": "PO"}', '')
+        project = kwargs['mock_project']
+        project.push(source=True, branch='some/b**r:a&n!ch')
+        self.assertDictEqual(project.url_info, {
+            'host': 'https://fake.com',
+            'project': 'example',
+            'resource': 'some-b-r-a-n-ch--enpo'
+        })
+
+    @fixture_mocked_project
+    @patch("txclib.project.logger.error")
+    def test_push_raises_authentication_exception(self, mock_logger, **kwargs):
+        project = kwargs['mock_project']
+        with self.assertRaises(AuthenticationError):
+            project.push()
+
+    @fixture_mocked_project
+    @patch("txclib.project.logger.error")
+    def test_delete_raises_authentication_exception(self, mock_logger,
+                                                    **kwargs):
+        project = kwargs['mock_project']
+        with self.assertRaises(AuthenticationError):
+            project.delete()
+
+    @fixture_mocked_project
+    @patch("txclib.project.logger.error")
+    @patch("txclib.utils.make_request")
+    def test_pull_raises_connection_exception(self, mock_request, mock_logger,
+                                              **kwargs):
+        """Test that all connection errors are properly handled."""
+        project = kwargs["mock_project"]
+        response = 502
+        msg = "Failed with code %d" % response
+        mock_request.side_effect = TXConnectionError(msg, code=response)
+        with self.assertRaises(TXConnectionError):
+            project.pull()
+
+    @fixture_mocked_project
+    @patch("txclib.project.logger.error")
+    @patch("txclib.utils.make_request")
+    def test_push_raises_connection_exception(self, mock_request, mock_logger,
+                                              **kwargs):
+        """Test that all connection errors are properly handled."""
+        project = kwargs["mock_project"]
+        response = 500
+        msg = "Failed with code %d" % response
+        mock_request.side_effect = TXConnectionError(msg, code=response)
+        with self.assertRaises(TXConnectionError):
+            project.push()
+
 
 class TestFormats(unittest.TestCase):
     """Tests for the supported formats."""
