@@ -16,69 +16,56 @@ adding code to this file you must take care of the following:
 """
 import os
 import re
-import shutil
 import sys
-
 try:
     import configparser
 except ImportError:
     import ConfigParser as configparser
-
-from six.moves import input
+from argparse import Namespace
 
 from txclib import utils, project
 from txclib.config import OrderedRawConfigParser
 from txclib.exceptions import UnInitializedError
 from txclib.parsers import delete_parser, help_parser, parse_csv_option, \
-    status_parser, pull_parser, set_parser, push_parser, init_parser
+    status_parser, pull_parser, set_parser, push_parser, init_parser, \
+    MAPPING, MAPPINGREMOTE, MAPPINGBULK
 from txclib.paths import posix_path
 from txclib.log import logger
+from txclib.wizard import Wizard
+from txclib import messages
 
 
 def cmd_init(argv, path_to_tx):
     """Initialize a new transifex project."""
     parser = init_parser()
-    (options, args) = parser.parse_args(argv)
-    if len(args) > 1:
-        parser.error("Too many arguments were provided. Aborting...")
-    if args:
-        path_to_tx = args[0]
-    else:
-        path_to_tx = os.getcwd()
+    options = parser.parse_args(argv)
+    path_to_tx = options.path_to_tx or os.getcwd()
 
+    print(messages.init_intro)
     save = options.save
     # if we already have a config file and we are not told to override it
     # in the args we have to ask
-    if os.path.isdir(os.path.join(path_to_tx, ".tx")) and not save:
-        logger.info("tx: There is already a tx folder!")
-        if not utils.confirm(
-            prompt='Do you want to delete it and reinit the project?',
-            default=False
-        ):
-            return
-        # Clean the old settings
-        # FIXME: take a backup
-        else:
-            save = True
-            rm_dir = os.path.join(path_to_tx, ".tx")
-            shutil.rmtree(rm_dir)
+    config_file = os.path.join(path_to_tx, ".tx", "config")
+    if os.path.isfile(config_file):
+        if not save:
+            if options.no_interactive:
+                parser.error("Project already initialized.")
+            logger.info(messages.init_initialized)
+            if not utils.confirm(messages.init_reinit):
+                return
+        os.remove(config_file)
 
-    logger.info("Creating .tx folder...")
-    os.mkdir(os.path.join(path_to_tx, ".tx"))
+    if not os.path.isdir(os.path.join(path_to_tx, ".tx")):
+        logger.info("Creating .tx folder...")
+        os.mkdir(os.path.join(path_to_tx, ".tx"))
 
     default_transifex = "https://www.transifex.com"
-    transifex_host = options.host or input("Transifex instance [%s]: " %
-                                           default_transifex)
+    transifex_host = options.host or default_transifex
 
-    if not transifex_host:
-        transifex_host = default_transifex
     if not transifex_host.startswith(('http://', 'https://')):
         transifex_host = 'https://' + transifex_host
 
-    config_file = os.path.join(path_to_tx, ".tx", "config")
     if not os.path.exists(config_file):
-        # The path to the config file (.tx/config)
-        logger.info("Creating skeleton...")
         # Handle the credentials through transifexrc
         config = OrderedRawConfigParser()
         config.add_section('main')
@@ -92,96 +79,106 @@ def cmd_init(argv, path_to_tx):
     prj = project.Project(path_to_tx)
     prj.getset_host_credentials(transifex_host, username=options.user,
                                 password=options.password,
-                                token=options.token, save=save)
+                                token=options.token, force=options.save,
+                                no_interactive=options.no_interactive)
     prj.save()
-    logger.info("Done.")
+
+    if not options.skipsetup and not options.no_interactive:
+        logger.info(messages.running_tx_set)
+        cmd_config([], path_to_tx)
+    else:
+        logger.info("Done.")
 
 
 def cmd_set(argv, path_to_tx):
+    """Add local or remote files under transifex. Warning: \
+This command will be deprecated in a future release of the \
+client. You should use the `tx config` command."""
+    cmd_config(argv, path_to_tx, is_legacy=True)
+
+
+def cmd_config(argv, path_to_tx, is_legacy=False):
     """Add local or remote files under transifex"""
-    parser = set_parser()
-    (options, args) = parser.parse_args(argv)
-
-    # Implement options/args checks
-    # TODO !!!!!!!
-    if options.local:
+    from_wizard = False
+    if len(argv) == 0:
+        # since interactive wizard should be equivalant to auto-local
+        # subcommand there are some default options that need to be set
+        default_options = {
+            'execute': True,
+            'subcommand': MAPPING,
+            'minimum_perc': 0,
+            'mode': None,
+        }
         try:
-            expression = args[0]
-        except IndexError:
-            parser.error("Please specify an expression.")
-        if not options.resource:
-            parser.error("Please specify a resource")
-        if not options.source_language:
+            # Run the wizard and configure parse with the wizard inputs
+            wizard_options = Wizard(path_to_tx).run()
+            wizard_options.update(default_options)
+            options = Namespace(**wizard_options)
+            parser = set_parser(is_legacy=is_legacy)
+            from_wizard = True
+        except SystemExit:
+            print("\n")
+            sys.exit(1)
+    else:
+        # hack to support backwards compatibility
+        # for --auto-local and --auto-remote options
+        if '--auto-local' in argv:
+            argv.pop(argv.index('--auto-local'))
+            argv.insert(0, MAPPING)
+        if '--auto-remote' in argv:
+            argv.pop(argv.index('--auto-remote'))
+            argv.insert(0, MAPPINGREMOTE)
+
+        is_subcommand = argv[0] in CONFIG_SUBCOMMANDS.keys()
+        parser = set_parser(subparser=is_subcommand, is_legacy=is_legacy)
+        options = parser.parse_args(argv)
+
+    _validate_set_arguments(parser, options)
+
+    if not hasattr(options, 'subcommand'):
+        bare_set(path_to_tx, options)
+    else:
+        try:
+            CONFIG_SUBCOMMANDS[options.subcommand](
+                path_to_tx, options, from_wizard=from_wizard
+            )
+        except KeyError:
+            parser.print_help()
+            sys.exit(2)
+
+
+def _validate_set_arguments(parser, options):
+    """
+    Do any extra required validation for set command arguments
+    """
+    if hasattr(options, 'is_source') and hasattr(options, 'language'):
+        if options.is_source and not options.language:
             parser.error("Please specify a source language.")
-        if '<lang>' not in expression:
-            parser.error("The expression you have provided is not valid.")
-        if not utils.valid_slug(options.resource):
-            parser.error("Invalid resource slug. The format is <project_slug>"
-                         ".<resource_slug> and the valid characters include"
-                         " [_-\w].")
-        _auto_local(path_to_tx, options.resource,
-                    source_language=options.source_language,
-                    expression=expression, source_file=options.source_file,
-                    execute=options.execute, regex=False)
-        if options.execute:
-            _set_minimum_perc(options.resource, options.minimum_perc,
-                              path_to_tx)
-            _set_mode(options.resource, options.mode, path_to_tx)
-            _set_type(options.resource, options.i18n_type, path_to_tx)
-        return
 
-    if options.remote:
-        try:
-            url = args[0]
-        except IndexError:
-            parser.error("Please specify an remote url")
-        _auto_remote(path_to_tx, url)
-        _set_minimum_perc(options.resource, options.minimum_perc, path_to_tx)
-        _set_mode(options.resource, options.mode, path_to_tx)
-        return
+    if hasattr(options, 'expression'):
+        if options.expression and '<lang>' not in options.expression:
+            parser.error("The expression you have provided is not valid.")
+
+    if hasattr(options, 'resource') and not\
+            utils.valid_resource_slug(options.resource):
+        parser.error("Invalid resource slug. The format is <project_slug>"
+                     ".<resource_slug> and the valid characters include "
+                     "[_-\w].")
+
+
+def bare_set(path_to_tx, options):
+    filename = options.filename
+    # Calculate relative path
+    path_to_file = os.path.relpath(filename, path_to_tx)
 
     if options.is_source:
         resource = options.resource
-        if not resource:
-            parser.error("You must specify a resource name with the "
-                         "-r|--resource flag.")
-
-        lang = options.language
-        if not lang:
-            parser.error("Please specify a source language.")
-
-        if len(args) != 1:
-            parser.error("Please specify a file.")
-
-        if not utils.valid_slug(resource):
-            parser.error("Invalid resource slug. The format is <project_slug>"
-                         ".<resource_slug> and the valid characters include "
-                         "[_-\w].")
-
-        file = args[0]
-        # Calculate relative path
-        path_to_file = os.path.relpath(file, path_to_tx)
         _set_source_file(path_to_tx, resource, options.language, path_to_file)
     elif options.resource or options.language:
         resource = options.resource
         lang = options.language
 
-        if len(args) != 1:
-            parser.error("Please specify a file")
-
-        # Calculate relative path
-        path_to_file = os.path.relpath(args[0], path_to_tx)
-
-        try:
-            _go_to_dir(path_to_tx)
-        except UnInitializedError as e:
-            utils.logger.error(e)
-            return
-
-        if not utils.valid_slug(resource):
-            parser.error("Invalid resource slug. The format is <project_slug>"
-                         ".<resource_slug> and the valid characters include "
-                         "[_-\w].")
+        _go_to_dir(path_to_tx)
         _set_translation(path_to_tx, resource, lang, path_to_file)
 
     _set_mode(options.resource, options.mode, path_to_tx)
@@ -189,7 +186,85 @@ def cmd_set(argv, path_to_tx):
     _set_minimum_perc(options.resource, options.minimum_perc, path_to_tx)
 
     logger.info("Done.")
-    return
+
+
+def subcommand_mapping(path_to_tx, options, from_wizard=False):
+    expression = options.expression
+    _auto_local(path_to_tx, options.resource,
+                source_language=options.source_language,
+                expression=expression, source_file=options.source_file,
+                execute=options.execute, regex=False)
+    if options.execute:
+        _set_minimum_perc(options.resource, options.minimum_perc,
+                          path_to_tx)
+        _set_mode(options.resource, options.mode, path_to_tx)
+        _set_type(options.resource, options.i18n_type, path_to_tx)
+
+    if from_wizard:
+        _print_instructions(options.resource, path_to_tx)
+
+
+def subcommand_mapping_remote(path_to_tx, options, **kwargs):
+    url = options.project_url
+    _auto_remote(path_to_tx, url)
+    _set_minimum_perc(None, options.minimum_perc, path_to_tx)
+    _set_mode(None, options.mode, path_to_tx)
+
+
+def _print_instructions(resource, path_to_tx):
+    keys = ['source_file', 'file_filter', 'source_lang', 'type']
+    prj = project.Project(path_to_tx)
+    fmt_kwargs = {k: prj.config.get(resource, k) for k in keys}
+    fmt_kwargs.update({'resource': resource})
+    print(messages.final_instr.format(**fmt_kwargs))
+
+
+def subcommand_mapping_bulk(path_to_tx, options, **kwargs):
+    """Add local files for multiple resources under transifex"""
+
+    if not options.file_extension.startswith('.'):
+        file_extension = '.{}'.format(options.file_extension)
+    else:
+        file_extension = options.file_extension
+
+    curpath = os.path.join(path_to_tx, options.source_file_dir)
+
+    for root, dirs, files in os.walk(curpath):
+        for file_name in files:
+            if not file_name.endswith(file_extension):
+                continue
+
+            source_file = os.path.join(
+                root, file_name
+            ).split(path_to_tx)[1][1:]
+            resource = source_file.split('.')[0].replace('/', '_')
+            relpath = root.split(options.source_file_dir)[1]
+
+            should_continue = True
+            for directory in options.ignore_dirs:
+                if relpath.strip('/').startswith(directory):
+                    should_continue = False
+                    break
+
+            if not should_continue:
+                break
+
+            expr = options.expression.format(
+                filepath=relpath, filename=file_name.split('.')[0],
+                extension=options.file_extension
+            ).replace('//', '/')
+
+            full_resource = '{}.{}'.format(options.project, resource)
+            _auto_local(
+                path_to_tx, full_resource, options.source_language, expr,
+                source_file=source_file, execute=options.execute, regex=False,
+            )
+
+            if options.execute:
+                _set_minimum_perc(full_resource, options.minimum_perc,
+                                  path_to_tx)
+                _set_mode(full_resource, options.mode, path_to_tx)
+                _set_type(full_resource, options.i18n_type, path_to_tx)
 
 
 def _auto_local(path_to_tx, resource, source_language, expression,
@@ -242,7 +317,7 @@ def _auto_local(path_to_tx, resource, source_language, expression,
             prj.config.get("%s" % resource, "source_file")
         except configparser.NoSectionError:
             raise Exception("No resource with slug \"%s\" was found.\nRun "
-                            "'tx set --auto-local -r %s \"expression\"' to "
+                            "'tx set auto-local -r %s \"expression\"' to "
                             "do the initial configuration." % resource)
 
     # Now let's handle the translation files.
@@ -332,7 +407,7 @@ def _auto_remote(path_to_tx, url):
 def cmd_push(argv, path_to_tx):
     """Push local files to remote server"""
     parser = push_parser()
-    (options, args) = parser.parse_args(argv)
+    options = parser.parse_args(argv)
     force_creation = options.force_creation
     languages = parse_csv_option(options.languages)
     resources = parse_csv_option(options.resources)
@@ -343,12 +418,13 @@ def cmd_push(argv, path_to_tx):
         parser.error("You need to specify at least one of the -s|--source, "
                      "-t|--translations flags with the push command.")
 
+    branch = get_branch_from_options(options, prj.root)
     prj.push(
         force=force_creation, resources=resources, languages=languages,
         skip=skip, source=options.push_source,
         translations=options.push_translations,
         no_interactive=options.no_interactive,
-        xliff=xliff
+        xliff=xliff, branch=branch
     )
     logger.info("Done.")
 
@@ -356,7 +432,7 @@ def cmd_push(argv, path_to_tx):
 def cmd_pull(argv, path_to_tx):
     """Pull files from remote server to local repository"""
     parser = pull_parser()
-    (options, args) = parser.parse_args(argv)
+    options = parser.parse_args(argv)
     if options.fetchall and options.languages:
         parser.error("You can't user a language filter along with the "
                      "-a|--all option")
@@ -368,19 +444,16 @@ def cmd_pull(argv, path_to_tx):
     skip = options.skip_errors
     minimum_perc = options.minimum_perc or None
 
-    try:
-        _go_to_dir(path_to_tx)
-    except UnInitializedError as e:
-        utils.logger.error(e)
-        return
+    _go_to_dir(path_to_tx)
 
     # instantiate the project.Project
     prj = project.Project(path_to_tx)
+    branch = get_branch_from_options(options, prj.root)
     prj.pull(
         languages=languages, resources=resources, overwrite=options.overwrite,
         fetchall=options.fetchall, fetchsource=options.fetchsource,
         force=options.force, skip=skip, minimum_perc=minimum_perc,
-        mode=options.mode, pseudo=pseudo, xliff=xliff
+        mode=options.mode, pseudo=pseudo, xliff=xliff, branch=branch
     )
     logger.info("Done.")
 
@@ -396,11 +469,7 @@ def _set_source_file(path_to_tx, resource, lang, path_to_file):
     if not lang:
         raise Exception("You haven't specified a source language.")
 
-    try:
-        _go_to_dir(path_to_tx)
-    except UnInitializedError as e:
-        utils.logger.error(e)
-        return
+    _go_to_dir(path_to_tx)
 
     if not os.path.exists(path_to_file):
         raise Exception("tx: File ( %s ) does not exist." %
@@ -447,11 +516,7 @@ def _set_translation(path_to_tx, resource, lang, path_to_file):
                         "project_slug.resource_slug." %
                         resource)
 
-    try:
-        _go_to_dir(path_to_tx)
-    except UnInitializedError as e:
-        utils.logger.error(e)
-        return
+    _go_to_dir(path_to_tx)
 
     # Warn the user if the file doesn't exist
     if not os.path.exists(path_to_file):
@@ -482,20 +547,20 @@ def _set_translation(path_to_tx, resource, lang, path_to_file):
 def cmd_status(argv, path_to_tx):
     """Print status of current project"""
     parser = status_parser()
-    (options, args) = parser.parse_args(argv)
+    options = parser.parse_args(argv)
     resources = parse_csv_option(options.resources)
     prj = project.Project(path_to_tx)
     resources = prj.get_chosen_resources(resources)
     resources_num = len(resources)
     for idx, res in enumerate(resources):
         p, r = res.split('.')
-        logger.info("%s -> %s (%s of %s)" % (p, r, idx + 1, resources_num))
-        logger.info("Translation Files:")
+        print("%s -> %s (%s of %s)" % (p, r, idx + 1, resources_num))
+        print("Translation Files:")
         slang = prj.get_resource_option(res, 'source_lang')
         sfile = prj.get_resource_option(res, 'source_file') or "N/A"
         lang_map = prj.get_resource_lang_mapping(res)
-        logger.info(" - %s: %s (%s)" % (utils.color_text(slang, "RED"),
-                    sfile, utils.color_text("source", "YELLOW")))
+        print(" - %s: %s (%s)" % (utils.color_text(slang, "RED"),
+              sfile, utils.color_text("source", "YELLOW")))
         files = prj.get_resource_files(res)
         fkeys = list(files.keys())
         fkeys.sort()
@@ -503,25 +568,23 @@ def cmd_status(argv, path_to_tx):
             local_lang = lang
             if lang in list(lang_map.values()):
                 local_lang = lang_map.flip[lang]
-            logger.info(" - %s: %s" % (utils.color_text(local_lang, "RED"),
-                        files[lang]))
-        logger.info("")
+            print(" - %s: %s" % (utils.color_text(local_lang, "RED"),
+                  files[lang]))
+        print("")
 
 
 def cmd_help(argv, path_to_tx):
     """List all available commands"""
     parser = help_parser()
-    (options, args) = parser.parse_args(argv)
-    if len(args) > 1:
-        parser.error("Multiple arguments received. Exiting...")
+    options = parser.parse_args(argv)
 
     # Get all commands
     fns = utils.discover_commands()
 
     # Print help for specific command
-    if len(args) == 1:
+    if options.command:
         try:
-            fns[argv[0]](['--help'], path_to_tx)
+            fns[options.command](['--help'], path_to_tx)
         except KeyError:
             utils.logger.error("Command %s not found" % argv[0])
     # or print summary of all commands
@@ -532,17 +595,17 @@ def cmd_help(argv, path_to_tx):
     keys = list(fns.keys())
     keys.sort()
 
-    logger.info("Transifex command line client.\n")
-    logger.info("Available commands are:")
+    print("Transifex command line client.\n")
+    print("Available commands are:")
     for key in keys:
-        logger.info("  %-15s\t%s" % (key, getattr(fns[key], '__doc__')))
-    logger.info("\nFor more information run %s command --help" % sys.argv[0])
+        print("  %-15s\t%s" % (key, getattr(fns[key], '__doc__')))
+    print("\nFor more information run %s command --help" % sys.argv[0])
 
 
 def cmd_delete(argv, path_to_tx):
     """Delete an accessible resource or translation in a remote server."""
     parser = delete_parser()
-    (options, args) = parser.parse_args(argv)
+    options = parser.parse_args(argv)
     languages = parse_csv_option(options.languages)
     resources = parse_csv_option(options.resources)
     skip = options.skip_errors
@@ -563,7 +626,7 @@ def _go_to_dir(path):
     """
     if path is None:
         raise UnInitializedError(
-            "Directory has not been initialzied. "
+            "Directory has not been initialized. "
             "Did you forget to run 'tx init' first?"
         )
     os.chdir(path)
@@ -600,3 +663,31 @@ def _set_project_option(resource, name, value, path_to_tx, func_name):
     prj = project.Project(path_to_tx)
     getattr(prj, func_name)(resources, value)
     prj.save()
+
+
+def get_branch_from_options(options, project_root):
+    """ Returns the branch name that needs to be used in command
+    based on parser options.
+
+    options: optparse parser options as returned from `parse()`
+    project_root: project root directory
+    """
+    if not options.branch:
+        return
+
+    if options.branch != '-1':
+        return options.branch
+
+    branch = utils.get_current_branch(project_root)
+    if not branch:
+        logger.error("You specified the --branch option but current "
+                     "directory does not seem to belong in any git repo.")
+        sys.exit(1)
+    return branch
+
+
+CONFIG_SUBCOMMANDS = {
+    MAPPING: subcommand_mapping,
+    MAPPINGREMOTE: subcommand_mapping_remote,
+    MAPPINGBULK: subcommand_mapping_bulk,
+}
