@@ -1,12 +1,10 @@
 # -*- coding: utf-8 -*-
 
-import getpass
 import os
 import re
 import fnmatch
 import datetime
 import time
-import sys
 import urllib3
 import six
 
@@ -21,20 +19,30 @@ try:
 except ImportError:
     import ConfigParser as configparser
 
+from requests.exceptions import HTTPError
+from slugify import slugify
+
 from txclib import web
+from txclib import api
 from txclib import utils
+from txclib import messages
 from urllib3.exceptions import SSLError
 from six.moves import input
+from txclib.exceptions import (
+    ConfigFileError, HttpNotFound, HttpNotAuthorized, MalformedConfigFile,
+    TransifexrcConfigFileError
+)
 from txclib.urls import API_URLS
-from txclib.config import OrderedRawConfigParser, Flipdict, CERT_REQUIRED
+from txclib.config import Flipdict, CERT_REQUIRED
 from txclib.log import logger
-from txclib.processors import visit_hostname
 from txclib.paths import posix_path, native_path, posix_sep
-from txclib.utils import confirm
+from txclib.utils import ProjectNotInit
 
 
-class ProjectNotInit(Exception):
-    pass
+DEFAULT_PULL_URL = 'pull_file'
+PULL_MODE_URL_NAME = "pull_{mode}_file"
+
+DEFAULT_API_HOSTNAME = "https://api.transifex.com"
 
 
 class Project(object):
@@ -53,18 +61,14 @@ class Project(object):
     def _init(self, path_to_tx=None):
         instructions = "Run 'tx init' to initialize your project first!"
         try:
-            self.root = self._get_tx_dir_path(path_to_tx)
-            self.config_file = self._get_config_file_path(self.root)
-            self.config = self._read_config_file(self.config_file)
+            self.root = utils.get_tx_dir_path(path_to_tx)
+            self.config_file = utils.get_config_file_path(self.root)
+            self.config = utils.read_config_file(self.config_file)
 
-            local_txrc_file = self._get_transifex_file(os.getcwd())
-            if os.path.exists(local_txrc_file):
-                self.txrc_file = local_txrc_file
-            else:
-                self.txrc_file = self._get_transifex_file()
-            self.txrc = self._get_transifex_config([self.txrc_file, ])
-        except ProjectNotInit as e:
-            logger.error('\n'.join([six.u(str(e)), instructions]))
+            self.txrc_file = utils.get_transifex_file()
+            self.txrc = utils.get_transifex_config(self.txrc_file)
+        except ProjectNotInit:
+            logger.error(instructions)
             raise
         host = self.config.get('main', 'host')
         if host.lower().startswith('https://'):
@@ -76,167 +80,95 @@ class Project(object):
         else:
             self.conn = urllib3.connection_from_url(host)
 
-    def _get_config_file_path(self, root_path):
-        """Check the .tx/config file exists."""
-        config_file = os.path.join(root_path, ".tx", "config")
-        logger.debug("Config file is %s" % config_file)
-        if not os.path.exists(config_file):
-            msg = "Cannot find the config file (.tx/config)!"
-            raise ProjectNotInit(msg)
-        return config_file
-
-    def _get_tx_dir_path(self, path_to_tx):
-        """Check the .tx directory exists."""
-        root_path = path_to_tx or utils.find_dot_tx()
-        logger.debug("Path to tx is %s." % root_path)
-        if not root_path:
-            msg = "Cannot find any .tx directory!"
-            raise ProjectNotInit(msg)
-        return root_path
-
-    def _read_config_file(self, config_file):
-        """Parse the config file and return its contents."""
-        config = OrderedRawConfigParser()
-        try:
-            config.read(config_file)
-        except Exception as err:
-            msg = "Cannot open/parse .tx/config file: %s" % err
-            raise ProjectNotInit(msg)
-        return config
-
-    def _get_transifex_config(self, txrc_files):
-        """Read the configuration from the .transifexrc files."""
-        txrc = OrderedRawConfigParser()
-        try:
-            txrc.read(txrc_files)
-        except Exception as e:
-            msg = "Cannot read configuration file: %s" % e
-            raise ProjectNotInit(msg)
-        self._migrate_txrc_file(txrc)
-        return txrc
-
-    def _migrate_txrc_file(self, txrc):
-        """Migrate the txrc file, if needed."""
-        if not os.path.exists(self.txrc_file):
-            return txrc
-        for section in txrc.sections():
-            orig_hostname = txrc.get(section, 'hostname')
-            hostname = visit_hostname(orig_hostname)
-            if hostname != orig_hostname:
-                msg = "Hostname %s should be changed to %s."
-                logger.info(msg % (orig_hostname, hostname))
-                if (sys.stdin.isatty() and sys.stdout.isatty() and
-                        utils.confirm('Change it now? ', default=True)):
-                    txrc.set(section, 'hostname', hostname)
-                    msg = 'Hostname changed'
-                    logger.info(msg)
-                else:
-                    hostname = orig_hostname
-            self._save_txrc_file(txrc)
-        return txrc
-
-    def _get_transifex_file(self, directory=None):
-        """Fetch the path of the .transifexrc file.
-        It is in the home directory of the user by default.
-        """
-        if directory is not None:
-            logger.debug(".transifexrc file is at %s" % directory)
-            return os.path.join(directory, ".transifexrc")
-
-        directory = os.path.expanduser('~')
-        txrc_file = os.path.join(directory, ".transifexrc")
-        logger.debug(".transifexrc file is at %s" % directory)
-        if not os.path.exists(txrc_file):
-            msg = "%s not found." % (txrc_file)
-            logger.info(msg)
-            mask = os.umask(0o077)
-            open(txrc_file, 'w').close()
-            os.umask(mask)
-            if os.path.exists(txrc_file):
-                logger.info('Created %s ' % txrc_file)
-            else:
-                logger.info('Could not create %s ' % txrc_file)
-        return txrc_file
-
     def validate_config(self):
         """To ensure the json structure is correctly formed."""
         pass
 
-    def getset_host_credentials(
-        self,
-        host,
-        username=None,
-        password=None,
-        token=None,
-        save=False
-    ):
+    def validate_credentials(self, username, password, host=None):
+        """Check if api credentials are valid."""
+        try:
+            api.Api(
+                username=username, password=password,
+                path_to_tx=self.txrc_file, host=host
+            ).get('auth_check')
+            return True
+        except HTTPError as e:
+            if e.response.status_code == 401:
+                return False
+            raise
+        return True
+
+    def getset_host_credentials(self, host, username=None, password=None,
+                                token=None, no_interactive=False,
+                                only_token=False, force=False):
         """Read .transifexrc and report user,
         pass or a token for a specific host else ask the user for input.
+        If the credentials provided are different from the .transifexrc file
+        ask for confirmation and update them
         """
-        # from_config is a flag that tells us if we got the credentials
-        # from the config file
-        from_config = False
-        # first check if a token has been given it should override everything
+
+        save = False
+        config_username, config_password = None, None
+        try:
+            config_username = self.txrc.get(host, 'username')
+            config_password = self.txrc.get(host, 'password')
+        except (configparser.NoOptionError, configparser.NoSectionError):
+            save = True
+
         if token:
             password = token
             username = 'api'
-        # if neither a token nor a username or a password were given
-        # try to get them from the rc file
-        elif not (username and password):
-            try:
-                username = self.txrc.get(host, 'username')
-                password = self.txrc.get(host, 'password')
-            except (configparser.NoOptionError, configparser.NoSectionError):
-                # if the rc has no credentials, we have to ask the user and
-                # update the rc file
-                save = True
-                # Ask the user if they have an api token
-                if confirm(
-                    prompt="\nDid you know that you can create an api"
-                    "token under your transifex user's settings?\n"
-                    "(Read more at https://docs.transifex.com/api/"
-                    "introduction#authentication)\n"
-                    "So, do you have an api token?",
-                    default=False
 
-                ):
-                    token_msg = "Please enter your api token: "
-                    while not token:
-                        token = input(token_msg)
-                    # Since we got a token, we use api as the username
-                    # and the token as the password
-                    username = 'api'
-                    password = token
-                else:
-                    username_msg = "Please enter your transifex username: "
-                    while not username:
-                        username = input(username_msg)
-                    while (not password):
-                        password = getpass.getpass()
+        if not (username and password) and not \
+               (config_username and config_password):
+            token = self._token_prompt(host)
+            username = 'api'
+            password = token
+            save = True
+        elif config_username and config_password:
+            if username == config_username and password == config_password:
+                pass
+            elif username and password:
+                if force:
+                    save = True
+                elif not no_interactive and utils.confirm(messages.update_txrc):  # noqa
+                    save = True
             else:
-                from_config = True
+                username = config_username
+                password = config_password
 
-        # lets see if there is a default username or a password
-        # unless we got the files from the config
-        if not from_config:
-            try:
-                username = self.txrc.get(host, 'username')
-                password = self.txrc.get(host, 'password')
-            except (configparser.NoOptionError, configparser.NoSectionError):
-                # if we do not have defaults, save the give credentials
-                save = True
+        # In the cases where we need to use the credentials with the new
+        # api we can only use a token and not a password so we do an extra
+        # validation and prompt the use for a token if the validation fails
+        if only_token and not self.validate_credentials(username, password):
+            logger.info("You need a valid api token to proceed")
+            username = 'api'
+            password = self._token_prompt(host)
+            save = True
 
         if save:
             logger.info("Updating %s file..." % self.txrc_file)
             if not self.txrc.has_section(host):
-                logger.info("No entry found for host %s. Creating..." % host)
                 self.txrc.add_section(host)
             self.txrc.set(host, 'username', username)
             self.txrc.set(host, 'password', password)
             self.txrc.set(host, 'hostname', host)
+            self.txrc.set(host, 'api_hostname',
+                          utils.DEFAULT_HOSTNAMES['api_hostname'])
             self.save()
         return username, password
 
+    def _token_prompt(self, host):
+        token = None
+        COLOR = "CYAN"
+        while not token:
+            token = input(utils.color_text(messages.token_msg, COLOR))
+            print(utils.color_text("Verifying token...", "YELLOW"))
+            if not self.validate_credentials('api', token, host):
+                logger.info(messages.token_validation_failed)
+                token = None
+        return token
+
     def set_remote_resource(self, resource, source_lang, i18n_type, host,
                             file_filter=None):
         """Method to handle the add/conf of a remote resource."""
@@ -248,11 +180,7 @@ class Project(object):
 
         p_slug, r_slug = resource.split('.', 1)
         file_filter = file_filter.replace("<sep>", r"%s" % posix_sep)
-        self.url_info = {
-            'host': host,
-            'project': p_slug,
-            'resource': r_slug
-        }
+        self._set_url_info(host=host, project=p_slug, resource=r_slug)
         extension = self._extension_for(i18n_type)[1:]
 
         self.config.set(resource, 'source_lang', source_lang)
@@ -282,11 +210,12 @@ class Project(object):
                 lang_map.update({k: v})
         except configparser.NoOptionError:
             pass
-        except (ValueError, KeyError):
-            raise Exception("Your lang map configuration is not correct.")
+        except ValueError:
+            raise MalformedConfigFile(
+                "Your lang map configuration is not correct.")
 
+        res_lang_map = Flipdict()
         if self.config.has_section(resource):
-            res_lang_map = Flipdict()
             try:
                 args = self.config.get(resource, "lang_map")
                 for arg in args.replace(' ', '').split(','):
@@ -294,8 +223,9 @@ class Project(object):
                     res_lang_map.update({k: v})
             except configparser.NoOptionError:
                 pass
-            except (ValueError, KeyError):
-                raise Exception("Your lang map configuration is not correct.")
+            except ValueError:
+                raise MalformedConfigFile(
+                    "Your lang map configuration is not correct.")
 
         # merge the lang maps and return result
         lang_map.update(res_lang_map)
@@ -348,7 +278,12 @@ class Project(object):
             for f_path in utils.files_in_project(self.root):
                 match = expr_rec.match(posix_path(f_path))
                 if match:
-                    lang = match.group(1)
+                    try:
+                        lang = match.group(1)
+                    except IndexError:
+                        msg = ("file_filter {} does not contain '<lang>' "
+                               "expresion".format(file_filter))
+                        raise MalformedConfigFile(msg)
                     if lang != source_lang:
                         f_path = os.path.relpath(f_path, self.root)
                         if f_path != source_file:
@@ -409,26 +344,8 @@ class Project(object):
         """Store the config dictionary
         in the .tx/config file of the project.
         """
-        self._save_tx_config()
-        self._save_txrc_file()
-
-    def _save_tx_config(self, config=None):
-        """Save the local config file."""
-        if config is None:
-            config = self.config
-        fh = open(self.config_file, "w")
-        config.write(fh)
-        fh.close()
-
-    def _save_txrc_file(self, txrc=None):
-        """Save the .transifexrc file."""
-        if txrc is None:
-            txrc = self.txrc
-        mask = os.umask(0o077)
-        fh = open(self.txrc_file, 'w')
-        txrc.write(fh)
-        fh.close()
-        os.umask(mask)
+        utils.save_tx_config(self.config_file, self.config)
+        utils.save_txrc_file(self.txrc_file, self.txrc)
 
     def get_full_path(self, relpath):
         if relpath[0] == os.path.sep:
@@ -440,32 +357,30 @@ class Project(object):
         pseudo_file = file_filter.replace('<lang>', '%s_pseudo' % slang)
         return native_path(pseudo_file)
 
+    def _slug_with_branch(self, resource_slug, branch):
+        """ Return the resource slug prefixed by the branch name.
+        """
+
+        return '{branch}--{resource}'.format(branch=slugify(branch),
+                                             resource=resource_slug)
+
     def pull(self, languages=[], resources=[], overwrite=True, fetchall=False,
              fetchsource=False, force=False, skip=False, minimum_perc=0,
-             mode=None, pseudo=False, xliff=False):
+             mode=None, pseudo=False, xliff=False, branch=None):
         """Pull all translations file from transifex server."""
         self.minimum_perc = minimum_perc
         resource_list = self.get_chosen_resources(resources)
         skip_decode = False
         params = {}
 
-        if mode == 'reviewed':
-            url = 'pull_reviewed_file'
-        elif mode == 'translator':
-            url = 'pull_translator_file'
-        elif mode == 'developer':
-            url = 'pull_developer_file'
-        elif mode == 'onlytranslated':
-            url = 'pull_onlytranslated_file'
-        elif mode == 'onlyreviewed':
-            url = 'pull_onlyreviewed_file'
-        else:
-            url = 'pull_file'
+        url = self._get_url_by_pull_mode(mode=mode)
 
         for resource in resource_list:
             logger.debug("Handling resource %s" % resource)
             self.resource = resource
             project_slug, resource_slug = resource.split('.', 1)
+            if branch:
+                resource_slug = self._slug_with_branch(resource_slug, branch)
             files = self.get_resource_files(resource)
             slang = self.get_resource_option(resource, 'source_lang')
             sfile = self.get_source_file(resource)
@@ -474,15 +389,28 @@ class Project(object):
             logger.debug("Language mapping is: %s" % lang_map)
             if mode is None:
                 mode = self._get_option(resource, 'mode')
-            self.url_info = {
-                'host': host,
-                'project': project_slug,
-                'resource': resource_slug
-            }
+            self._set_url_info(host=host, project=project_slug,
+                               resource=resource_slug)
             logger.debug("URL data are: %s" % self.url_info)
 
-            stats = self._get_stats_for_resource()
-            details_response, _ = self.do_url_request('resource_details')
+            try:
+                stats = self._get_stats_for_resource()
+                details_response, _ = self.do_url_request('resource_details')
+            except Exception as e:
+                if isinstance(e, HttpNotAuthorized):
+                    logger.error("Request is not authorized.")
+                    continue
+                if isinstance(e, HttpNotFound):
+                    msg = "Resource %s doesn't exist on the server."
+                    logger.error(msg % resource)
+                    continue
+                if isinstance(e, SSLError) or not skip:
+                    raise
+                else:
+                    msg = "Skipping resource %s"
+                    logger.warn(msg % resource)
+                    continue
+
             details = utils.parse_json(details_response)
             if details['i18n_type'] in self.SKIP_DECODE_I18N_TYPES:
                 skip_decode = True
@@ -535,6 +463,8 @@ class Project(object):
             if pull_languages:
                 logger.debug("Pulling languages for: %s" % pull_languages)
                 msg = "Pulling translations for resource %s (source: %s)"
+                if branch:
+                    msg += " for branch {}".format(branch)
                 if xliff:
                     msg += " [xliff format]"
                 logger.info(msg % (resource, sfile))
@@ -646,7 +576,7 @@ class Project(object):
 
     def push(self, source=False, translations=False, force=False,
              resources=[], languages=[], skip=False, no_interactive=False,
-             xliff=False):
+             xliff=False, branch=None):
         """Push all the resources"""
         resource_list = self.get_chosen_resources(resources)
         self.skip = skip
@@ -659,6 +589,8 @@ class Project(object):
         for resource in resource_list:
             push_languages = []
             project_slug, resource_slug = resource.split('.', 1)
+            if branch:
+                resource_slug = self._slug_with_branch(resource_slug, branch)
             files = self.get_resource_files(resource, xliff=xliff)
             slang = self.get_resource_option(resource, 'source_lang')
             sfile = self.get_source_file(resource)
@@ -666,24 +598,24 @@ class Project(object):
             host = self.get_resource_host(resource)
             logger.debug("Language mapping is: %s" % lang_map)
             logger.debug("Using host %s" % host)
-            self.url_info = {
-                'host': host,
-                'project': project_slug,
-                'resource': resource_slug
-            }
+            self._set_url_info(host=host, project=project_slug,
+                               resource=resource_slug)
 
-            logger.info("Pushing translations for resource %s:" % resource)
+            message = "Pushing resource {}".format(resource)
+            if branch:
+                message += " for branch {}".format(branch)
+            logger.info(message)
 
             stats = self._get_stats_for_resource()
 
             if force and not no_interactive:
-                answer = input("Warning: By using --force, the uploaded "
-                               "files will overwrite remote translations, "
-                               "even if they are newer than your uploaded "
-                               "files.\nAre you sure you want to continue? "
-                               "[y/N]")
+                msg = ("Warning: By using --force, the uploaded files will "
+                       "overwrite remote translations, even if they are newer "
+                       "than your uploaded files.\nAre you sure you want to "
+                       "continue?")
 
-                if answer not in ["", 'Y', 'y', "yes", 'YES']:
+                if not utils.confirm(prompt=msg, default=False):
+                    logger.info("Aborting...")
                     return
 
             if source:
@@ -695,13 +627,14 @@ class Project(object):
                     continue
                 # Push source file
                 try:
-                    logger.warning("Pushing source file (%s)" % sfile)
+                    logger.info("Pushing source file (%s)" % sfile)
                     if not self._resource_exists(stats):
                         logger.info("Resource does not exist.  Creating...")
                         fileinfo = "%s;%s" % (resource_slug, slang)
                         filename = self.get_full_path(sfile)
                         self._create_resource(
-                            resource, project_slug, fileinfo, filename
+                            resource, project_slug, fileinfo, filename,
+                            branch=branch
                         )
                     self.do_url_request(
                         'push_source', multipart=True, method="PUT",
@@ -711,7 +644,12 @@ class Project(object):
                         params=params,
                     )
                 except Exception as e:
-                    if isinstance(e, SSLError) or not skip:
+                    if isinstance(e, SSLError):
+                        raise
+                    elif not skip:
+                        logger.error("Could not upload source file. "
+                                     "You can use --skip to ignore this "
+                                     "error and continue the execution.")
                         raise
                     else:
                         logger.error(e)
@@ -719,16 +657,15 @@ class Project(object):
                 try:
                     self.do_url_request('resource_details')
                 except Exception as e:
-                    if isinstance(e, SSLError):
-                        raise
-                    code = getattr(e, 'code', None)
-                    if code == 401:
+                    if isinstance(e, HttpNotAuthorized):
                         logger.error("Request is not authorized.")
                         continue
-                    if code == 404:
+                    if isinstance(e, HttpNotFound):
                         msg = "Resource %s doesn't exist on the server."
                         logger.error(msg % resource)
                         continue
+                    if isinstance(e, SSLError) or not skip:
+                        raise
 
             if translations:
                 # Check if given language codes exist
@@ -736,15 +673,11 @@ class Project(object):
                     push_languages = list(files.keys())
                 else:
                     push_languages = []
-                    f_langs = list(files.keys())
                     for l in languages:
                         if l in list(lang_map.keys()):
                             l = lang_map[l]
                         push_languages.append(l)
-                        if l not in f_langs:
-                            msg = "Warning: No mapping found for "
-                            "language code '%s'."
-                            logger.error(msg % utils.color_text(l, "RED"))
+
                 logger.debug("Languages to push are %s" % push_languages)
 
                 # Push translation files one by one
@@ -755,7 +688,12 @@ class Project(object):
                     else:
                         remote_lang = lang
 
-                    local_file = files[local_lang]
+                    local_file = files.get(local_lang)
+                    if not local_file:
+                        msg = ("Warning: No local translation file found for "
+                               "language code '%s'.")
+                        logger.error(msg % utils.color_text(lang, "RED"))
+                        continue
 
                     kwargs = {
                         'lang': remote_lang,
@@ -785,11 +723,25 @@ class Project(object):
                         )
                         logger.debug("Translation %s pushed." % remote_lang)
                     except utils.HttpNotFound:
-                        if not source:
-                            logger.error("Resource hasn't been created. "
+                        # If the resource exists, we suppose the language code
+                        # was invalid
+                        if self._resource_exists(stats):
+                            error_msg = ("Error: '%s' is not a valid"
+                                         " language code. Please refer to"
+                                         " https://www.transifex.com/"
+                                         "languages/ for a list of supported"
+                                         " languages in Transifex." % lang)
+                        else:
+                            error_msg = ("Resource hasn't been created. "
                                          "Try pushing source file.")
+                        logger.error(error_msg)
                     except Exception as e:
-                        if isinstance(e, SSLError) or not skip:
+                        if isinstance(e, SSLError):
+                            raise
+                        elif not skip:
+                            logger.error("Could not push translations. "
+                                         "You can use --skip to ignore this "
+                                         "error and continue the execution.")
                             raise
                         else:
                             logger.error(e)
@@ -808,13 +760,22 @@ class Project(object):
         for resource in resource_list:
             project_slug, resource_slug = resource.split('.', 1)
             host = self.get_resource_host(resource)
-            self.url_info = {
-                'host': host,
-                'project': project_slug,
-                'resource': resource_slug
-            }
+            self._set_url_info(host=host, project=project_slug,
+                               resource=resource_slug)
             logger.debug("URL data are: %s" % self.url_info)
-            json, _ = self.do_url_request('project_details', project=self)
+            try:
+                json, _ = self.do_url_request('project_details', project=self)
+            except Exception as e:
+                if isinstance(e, HttpNotAuthorized):
+                    logger.error("Request is not authorized.")
+                    continue
+                if isinstance(e, HttpNotFound):
+                    msg = "Resource %s doesn't exist on the server."
+                    logger.error(msg % resource)
+                    continue
+                if isinstance(e, SSLError) or not skip:
+                    raise
+
             project_details = utils.parse_json(json)
             stats = self._get_stats_for_resource()
             delete_func(project_details, resource, stats, languages)
@@ -895,7 +856,7 @@ class Project(object):
             self.do_url_request(
                 'delete_translation', language=language, method="DELETE"
             )
-            msg = "Deleted language %s from resource %s of project %s."
+            msg = "Deleted %s translations of resource %s of project %s."
             logger.info(msg % (language, resource_slug, project_slug))
         except Exception as e:
             msg = "Unable to delete translation %s"
@@ -913,7 +874,7 @@ class Project(object):
         try:
             hostname = self.txrc.get(host, 'hostname')
         except configparser.NoSectionError:
-            raise Exception(
+            raise TransifexrcConfigFileError(
                 "No entry found for host %s. Edit"
                 " ~/.transifexrc and add the appropriate"
                 " info in there." % host
@@ -1282,7 +1243,9 @@ class Project(object):
             res = utils.parse_json(json)
             return res[i18n_type]['file-extensions'].split(',')[0]
         except Exception as e:
-            logger.error(e)
+            logger.warning(
+                "The file extension for i18n_type %s is not found." % str(e)
+            )
             return ''
 
     def _resource_exists(self, stats):
@@ -1315,9 +1278,10 @@ class Project(object):
             passwd = self.txrc.get(host, 'password')
             hostname = self.txrc.get(host, 'hostname')
         except configparser.NoSectionError:
-            raise Exception("No user credentials found for host %s. Edit "
-                            "~/.transifexrc and add the appropriate "
-                            "info in there." % host)
+            raise TransifexrcConfigFileError(
+                "No user credentials found for host %s. Edit "
+                "~/.transifexrc and add the appropriate "
+                "info in there." % host)
 
         # Create the Url
         kwargs['hostname'] = hostname
@@ -1327,7 +1291,7 @@ class Project(object):
 
         i18n_type = self._get_option(resource, 'type')
         if i18n_type is None:
-            raise Exception(
+            raise ConfigFileError(
                 "Please define the resource type in "
                 ".tx/config (eg. type = PO). "
                 "More info: http://bit.ly/txcconfig"
@@ -1340,12 +1304,30 @@ class Project(object):
             "uploaded_file": (name, open(filename, 'rb').read()),
             "i18n_type": i18n_type
         }
+        if kwargs.get('branch'):
+            data.update({'category': kwargs['branch']})
 
         r, charset = utils.make_request(
             method, hostname, url, username, passwd, data
         )
         return r
 
+    def _get_url_by_pull_mode(self, mode):
+        """Get the url by the pull mode.
+
+        If the pull mode is not valid, the default pull mode will be used.
+        """
+        if mode is not None:
+            try:
+                # check if specific url name exists in the url list
+                API_URLS[PULL_MODE_URL_NAME.format(mode=mode)]
+                return PULL_MODE_URL_NAME.format(mode=mode)
+            except KeyError:
+                logger.warning('Invalid mode provided. ' +
+                               'Default pull mode will be used')
+
+        return DEFAULT_PULL_URL
+
     def _get_option(self, resource, option):
         """Get the value for the option in the config file.
 
@@ -1396,3 +1378,10 @@ class Project(object):
             file_content = file_content.encode(charset)
         fd.write(file_content)
         fd.close()
+
+    def _set_url_info(self, host, project, resource):
+        self.url_info = {
+            'host': host,
+            'project': project,
+            'resource': resource
+        }
